/***************************************************************************

	Kuraudo.signals

********
Overview
********

	This library provides a useful mechanism
	known as "signals & slots." Essentially, what
	it allows you to do is connect events ("signals")
	to recipient procs ("slots"). One signal may be
	associated with as many slots as you like. When you
	emit the signal, all of the associated slots are
	called (in the order they were connected) with
	the arguments specified when emitting the signal.

***************
Getting Started
***************

	To begin using signals and slots, give the type
	you wish to emit signals a variable of type
	/signal_mgr. For example:

		mob/var/signal_mgr/signals = new

******************
Connecting Signals
******************

	Start connecting signals to slots with the signal_mgr
	connect() proc. It looks like so:

		proc/connect(signal, destination, slot)

	signal:
		This is the signal. It can be any kind of variable,
		but if you use, say, an /obj, you must remember
		to use the exact same one when emitting the signal!
		I'd suggest using strings or numbers.

	destination:
		This is the object receiving the signal. For
		example, if a switch connects a "flipped" signal
		to a slot belonging to a door (say, to unlock it),
		the door object would be the destination. It must
		be an actual object reference (not a type!)

	slot:
		This is the proc belonging to "destination" to
		be called when you emit your signal. There are
		two ways to pass the slot argument:

			path: /obj/door/proc/set_unlocked
			string: "set unlocked"

		Notice that, if there is an underscore in the proc
		path, there will be a space in the string version
		(as of BYOND v.446)


	So, an example call to connect might look like so:

		lock_switch.signals.connect("flipped", \
			some_door, "set unlocked")

*********************
Disconnecting Signals
*********************

	You can disconnect the same way:

		lock_switch.signals.disconnect("flipped", \
			some_door, "set unlocked")

	Since a single signal may be received by numerous
	slots, you must specify all 3 arguments to the
	disconnect proc. Note also that if you connected
	with a path, you can't disconnect with a string,
	and vice versa. The call to disconnect() must be
	identical to the connect() call. For this reason,
	it's a good idea to pick either proc name arguments
	or proc path arguments and be consistent throughout
	your project.

	signal_mgr also contains a clear() proc to remove
	ALL signal-slot connections at one time:

		lock_switch.signals.clear()

****************
Emitting Signals
****************

	Finally, you can emit signals using the signal_mgr.emit() proc.
	It looks like so:

		proc/emit(signal, ...)

	The first argument is the signal to emit. The remaining arguments
	are what you pass to the slots. For example:

		signals.emit("SIGNAL!", arg1, arg2)

	Should be received by a slot which looks like this:

		proc/some_slot(arg1, arg2)

****************
Blocking Signals
****************

	Sometimes, you'd like to do things that emit signals without notifying
	the slots, and disconnecting/reconnecting all of the slots is more of
	a hassle than it's worth. To temporarily disable a /signal_mgr from
	emitting signals, use the blockSignals() proc, or to check the status
	of blocked signals use the signalsBlocked() proc. blockSignals() returns
	the previous value of signalsBlocked(), so you can safely use it in
	nested blocking code by restoring it using the return value:

		var/previously_blocked = signals.blockSignals(TRUE)
		// other stuff
		signals.blockSignals(previously_blocked)

	signalsBlocked() returns a false value (0, null, empty string) if signals
	are not currently blocked, and true (not 0, null, or empty string)
	if signals will be emitted. When using /signal_mgr/blockSignals(), it
	is recommended to use DM's TRUE and FALSE macros, as above.

***************************************************************************/